# -----------------------------------------------------------------------------
# efftcl_extra.tcl --
#
#       A package of extra procedures to add to the efftcl package, which
#       accompanies the following book:
#
# ----------------------------------------------------------------------
#  Effective Tcl/Tk Programming
#    Mark Harrison, DSC Communications Corp.
#    Michael McLennan, Bell Labs Innovations for Lucent Technologies
#    Addison-Wesley Professional Computing Series
# ======================================================================
#  Copyright (c) 1996-1997  Lucent Technologies Inc. and Mark Harrison
# ======================================================================
# -----------------------------------------------------------------------------
#   TSIPP Workbench - An interactive program to create 3D images, animations,
#   and Tk widgets using TSIPP, a graphical toolkit developed by Mark Diekhans.
#   Copyright (C) 2001 Paul D Welton
#   15 Grengold Way, Kanata, ON, Canada, K2T 1C8 - paul.welton@sympatico.ca
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# -----------------------------------------------------------------------------
# Revision:   1.4   : First publically announced release.
# -----------------------------------------------------------------------------

package require Efftcl

package provide Efftcl_extra 1.0

#  As Efftcl does not use namespaces, they are not used here for consistency.

# -----------------------------------------------------------------------------
# colordial_get_hsb --
#
#       The base colordial exports its state in RBG format, and provides no
#       means to reload the widget with a previous state.  As the translation
#       from RGB back to HSB is non-trivial and may result in loss of
#       precision, the following routine allows the state to be extracted in
#       HSB format.
#
# Arguments:
#       win     - pathname of the colordial widget
#
# Results:
#       A list of the current HSB values represented by the specified widget.

proc colordial_get_hsb {win} {
    global cdInfo

    set h $cdInfo($win-hue)
    set s $cdInfo($win-saturation)
    set v $cdInfo($win-brightness)

    return [list $h $s $v]
}

# -----------------------------------------------------------------------------
# colordial_set_hsb --
#
#       Companion procedure to "colordial_get_hsb", which allows the state to
#       be reloaded in HSB format, and the graphical representation to be
#       refreshed to reflect that state.
#
# Arguments:
#       win     - pathname of the colordial widget
#       h, s, b - Hue Saturation and Brightness values as discrete arguments.
#
# Results:
#       No return value.

proc colordial_set_hsb {win h s b} {
    global cdInfo

    set cdInfo($win-hue)        $h
    set cdInfo($win-saturation) $s
    set cdInfo($win-brightness) $b

    colordial_refresh $win
    return
}

# -----------------------------------------------------------------------------
# colordial_state --
#
#       This procedure sets the state of a specified color_dial to either
#       disabled or normal.  When disabled, the control handles become
#       line rather than solid items and the bindings to them are removed.
#
# Arguments:
#       win     - pathname of the colordial widget
#       h, s, b - Hue Saturation and Brightness values as discrete arguments.
#
# Results:
#       No return value.

proc colordial_state {win state} {
    global cdInfo

    if {![info exists cdInfo($win-state)] ||                 \
        ![string equal $state $cdInfo($win-state)]} {
        switch $state {

        disabled {
            #  Capture and store the bindings so that they can be restored
            #  when the state is returned to normal.
            set cdInfo($win-hsval_binding) [$win.dial bind hsval <B1-Motion>]
            set cdInfo($win-bval_binding)  [$win.dial bind bval  <B1-Motion>]

            #  Set the bindings to do nothing.
            $win.dial bind hsval <B1-Motion> { }
            $win.dial bind bval  <B1-Motion> { }

            #  Give the colordial the appearance of being disabled.
            $win.dial itemconfigure hsval -fill {}
            $win.dial itemconfigure bval  -fill {}
        }

        normal {
            #  Restore the bindings.
            $win.dial bind hsval <B1-Motion> $cdInfo($win-hsval_binding)
            $win.dial bind bval  <B1-Motion> $cdInfo($win-bval_binding)

            #  Restore the active appearance of the colordial.
            $win.dial itemconfigure hsval -fill black
            $win.dial itemconfigure bval  -fill black
        }

        default {
            error "Unknown state \"$state\""
        }

        }

        set cdInfo($win-state) $state
    }

    return
}

# -----------------------------------------------------------------------------
# END OF PACKAGE "Efftcl_extra"
# -----------------------------------------------------------------------------



