# -----------------------------------------------------------------------------
# testProcs.tcl --
#
#       Test procedures for the TSIPP Workbench.
# -----------------------------------------------------------------------------
#   TSIPP Workbench - An interactive program to create 3D images, animations,
#   and Tk widgets using TSIPP, a graphical toolkit developed by Mark Diekhans.
#   Copyright (C) 2001 Paul D Welton
#   15 Grengold Way, Kanata, ON, Canada, K2T 1C8 - paul.welton@sympatico.ca
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# -----------------------------------------------------------------------------
# Revision:   1.4   : First publically announced release.
# -----------------------------------------------------------------------------

#  Redefine the file dialogs to use filenames supplied by the test procedure
#  in the global variable ::G_filename.
if {[info commands tk_getOpenFile_original] == ""} {
    uplevel #0 rename tk_getOpenFile tk_getOpenFile_original
    uplevel #0 [list proc tk_getOpenFile {args} {
        return $::G_filename
    }]
    debugPrint "Replaced tk_getOpenFile"
}

if {[info commands tk_getSaveFile_original] == ""} {
    uplevel #0 rename tk_getSaveFile tk_getSaveFile_original
    uplevel #0 [list proc tk_getSaveFile {args} {
        return $::G_filename
    }]
    debugPrint "Replaced tk_getSaveFile"
}

# -----------------------------------------------------------------------------
# compareFiles --
#
#       This procedure loads compares two files and reports differences.  It
#       is assumed that each line of each file is a well formed list, and that
#       the lists on corresponding lines are identical except that numerical
#       values may differ by very small amounts due to rounding errors.
#
# Arguments:
#       l1, l2    - lists.
#       tolerance - acceptable variation between numerical values with
#                   respect to the sum of their absolute values.
#       msg       - name of a string of the caller to which diagnostics should
#                   be appended.
#
# Results:
#       Returns 1 if the lists do not match and zero otherwise.

proc acceptRoundingErrors {l1 l2 tolerance pMsg} {
    upvar $pMsg msg

    set err 0
    foreach a $l1 b $l2 {
        if {![string equal $a $b]} {
            if {([llength $a] > 1) && ([llength $b] > 1)} {
                set err [expr $err || [acceptRoundingErrors $a $b $tolerance msg]]
            } else {
                if {[catch {expr $a != $b} result]} {
                    append msg "\nError comparing $a and $b - $result"
                    set err 1
                } elseif {$result} {
                    if {[catch {expr abs (($a - $b) / (abs ($a) + abs ($b))) > $tolerance} result]} {
                        append msg "\nError comparing $a and $b - $result"
                        set err 1
                    } elseif {$result} {
                        append msg "\nNumerics $a and $b differ by more than $tolerance"
                        set err 1
                    }
                }
            }
        }
    }

    return $err
}

# -----------------------------------------------------------------------------
# compareFiles --
#
#       This procedure loads compares two files and reports differences.  It
#       is assumed that each line of each file is a well formed list, and that
#       the lists on corresponding lines are identical except that numerical
#       values may differ by very small amounts due to rounding errors.
#
# Arguments:
#       f1, f2 - filenames.
#
# Results:
#       Returns 1 if the files do not match and zero otherwise.

proc compareFiles {f1 f2} {

    set fp1 [open $f1]
    set fp2 [open $f2]

    set lineNumber 0
    set mismatch   0

    while {![eof $fp1] && ![eof $fp2]} {
        gets $fp1 l1
        gets $fp2 l2
        incr lineNumber

        if {[acceptRoundingErrors $l1 $l2 0.0000001 msg]} {
            debugPrint [string repeat {=} 72]
            debugPrint "Problems comparing lines $lineNumber:"
            debugPrint "File \"$f1\": $l1"
            debugPrint "File \"$f2\": $l2"
            debugPrint [string repeat {-} 72]
            debugPrint $msg
            set mismatch 1
            unset msg
        }
    }

    if {![eof $fp1] && [eof $fp2]} {
        debugPrint "File \"$f2\" ended at line $lineNumber but \"$f1\" continues."
    }

    if {[eof $fp1] && ![eof $fp2]} {
        debugPrint "File \"$f1\" ended at line $lineNumber but \"$f2\" continues."
    }

    close $fp1
    close $fp2

    if {$mismatch} {
        debugPrint [string repeat {=} 72]
    } else {
        debugPrint "Files \"$f1\" and \"$f2\" compare ok for $lineNumber lines"
    }

    return $mismatch
}

# -----------------------------------------------------------------------------
# loadSaveTest --
#
#       This procedure loads, renders and saves each source file in the
#       current working directory, and then compares the saved file with
#       the previous version.  On completion of the test, a summary of how
#       many saved files matched the original is generated.
#
# Arguments:
#       None.
#
# Results:
#       No return value.

proc loadSaveTest {} {

    set nTest     0
    set nFailures 0

    foreach filename [glob *.tcl] {
        regsub {\.tcl$} $filename {_test.tcl} ::G_filename
        exec cp $filename $::G_filename

        debugPrint "Loading and Saving $::G_filename ..."
        .g_win.fileControl.e.load   invoke
        debugPrint " ... loaded $::G_filename"
        update
        .g_win.renderControl.e.render invoke
        debugPrint " ... rendered $::G_filename"
        update
        .g_win.fileControl.e.saveAs invoke
        debugPrint " ... saved $::G_filename"

        set FAILED [compareFiles $::G_filename ${::G_filename}%]

        incr nTest
        incr nFailures $FAILED

        exec mv $::G_filename ${filename}-test
    }

    debugPrint "In $nTest tests, there were $nFailures failures."
}

# -----------------------------------------------------------------------------
# END OF FILE "testProcs.tcl"
# -----------------------------------------------------------------------------
